#!/usr/bin/perl -w
#=========================================================================== 
# Organization : Shanghai Fudan Microelectronics Co.,Ltd. 
# Filename     : gen_verilog_tree.pl
# Author       : Feng Bohan
# Create Time  : 13:40:46 2024-05-03
# Last Modified: 14:53:58 2024-05-03
# Abstract     :  
#--------------------------------------------------------------------------
# Description:
# 
#--------------------------------------------------------------------------
# Modification History:
#--------------------------------------------------------------------------
# Rev      Date        Who         Description
# ---      ----        ---         -----------
# 0.0.01   2024-05-03  Feng Bohan  initial version
#===========================================================================
use strict;
use warnings;
use Verilog::Netlist;
use Verilog::Getopt;

my $verilog_file;
my $target_mod_name;
if(defined $ARGV[0] and defined $ARGV[1]){
    print "process file: $ARGV[0]\n";
    $verilog_file = $ARGV[0];
    $target_mod_name = $ARGV[1];
}
else {
    die "\e[31minput error; eg: $0 verilog_file.\n\e[0m";
}

my $opt = new Verilog::Getopt;
$opt->parameter( "+incdir+verilog",
                 "-y","verilog",
                 );
my $nl = new Verilog::Netlist(options => $opt,);
$nl->read_file(filename=>$verilog_file);
$nl->link();

my $top = $nl->find_module("top");
# $top->is_top(1);
foreach my $mod ($nl->modules_sorted) {
    if($mod->is_top){
        my $name = $mod->name;
        print "find modules: $name\n";
    }
}
my @mod_path;
get_hier_by_mod($top, $target_mod_name);

sub get_hier_by_mod{
    my ($top, $target_mod_name) = @_;

    my $hdl_path = $top->name;
    get_cells($top, $target_mod_name);

    my $hier_path;
    if(scalar(@mod_path) == 1){
        $hier_path = $mod_path[0];
    }
    else{
        $hier_path = user_select(\@mod_path)
    }
    print "hier_path = $hier_path\n";
}

sub user_select{
    my ($data_array) = @_; 

    # 打印数组供用户选择
    print "请选择一个选项:\n";
    for (my $i = 0; $i < scalar(@$data_array); $i++) {
        print "$i. $data_array->[$i]\n";
    }

    # 读取用户输入
    print "请输入选项编号: ";
    chomp(my $user_choice = <STDIN>);

    # 验证用户输入是否有效
    if ($user_choice >= 0 && $user_choice < scalar(@$data_array)) {
        print "您选择了: $data_array->[$user_choice]\n";
    } else {
        print "无效的选择，请输入有效的选项编号。\n";
    }
    return $data_array->[$user_choice];
}


sub get_cells{
    my ($mod, $hdl_path) = @_;
    foreach my $cell ($mod->cells_sorted){
        my $local_hdl_path = $hdl_path."/".$cell->name;
        if($cell->submodname eq $target_mod_name){
            push @mod_path, $local_hdl_path;
            print "get_cells: ".$local_hdl_path."\n";
        }
        get_mods($cell, $local_hdl_path);
    }
}

sub get_mods{
    my ($cell, $hdl_path) = @_;
    foreach my $mod ($cell->submod){
        # print "get_mods: ".$mod->name."\n";
        get_cells($mod, $hdl_path);
    }
} 
